"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Query = void 0;
class Query {
    constructor(prisma) {
        this.checkForCallback = function (cb, keys) {
            return (typeof cb === 'function')
                ? cb(keys)
                : cb;
        };
        this.clauseCallback = function (id) {
            return {
                tags: {
                    some: {
                        id: id
                    }
                }
            };
        };
        this.prisma = prisma;
        this.sheets = [];
        this.keys = {};
        this.result = {};
        this.points = [];
        this.grid = {};
    }
    get(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const ids = yield this.getTagIds(tags);
            let clause = this.clauseCallback(ids[0]);
            for (let i in ids) {
                if (Number(i) > 0) {
                    this.setNestedClause(clause, ids[i]);
                }
            }
            let sheets = yield this.prisma.sheet.findMany({
                where: clause,
                include: {
                    tags: true
                }
            });
            this.parseSheets(sheets);
            this.setDataPoints();
            return this;
        });
    }
    parseSheets(sheets) {
        this.sheets = sheets.map(sheet => {
            return {
                id: sheet.id,
                tags: sheet.tags.map(tag => {
                    return {
                        id: tag.id,
                        value: tag.name,
                        categoryId: tag.categoryId
                    };
                }),
                rows: JSON.parse(sheet.rows),
                columns: JSON.parse(sheet.columns),
                data: JSON.parse(sheet.data),
                meta: JSON.parse(sheet.meta),
            };
        });
    }
    setDataPoints() {
        this.sheets.forEach(sheet => {
            sheet.tags.forEach(tag => {
                if (tag.categoryId) {
                    this.addKey(String(tag.categoryId), tag.value);
                }
            });
            sheet.data.forEach((points, r) => {
                points.forEach((point, c) => {
                    this.points.push({
                        tags: sheet.tags,
                        row: sheet.rows[r],
                        column: sheet.columns[c],
                        value: point,
                        meta: (sheet.meta) ? sheet.meta[r][c] : null,
                    });
                    this.addKey('row', sheet.rows[r]);
                    this.addKey('column', sheet.columns[c]);
                });
            });
        });
    }
    addKey(category, value) {
        if (!this.keys[category]) {
            this.keys[category] = {};
        }
        this.keys[category][value] = true;
    }
    merge(grid) {
        let rows = this.checkForCallback(grid.rows, this.keys);
        let columns = this.checkForCallback(grid.columns, this.keys);
        let points = [];
        rows.forEach((rowCb, r) => {
            let rowCbResult = rowCb(this.points);
            points[r] = rowCbResult.points;
            let rowKey = rowCbResult.label;
            this.result[rowKey] = {};
            columns.forEach((columnCb, c) => {
                let cellPoints = columnCb(points[r]);
                let colKey = cellPoints.label;
                let cell = grid.cell(cellPoints.points);
                this.result[rowKey][colKey] = cell;
            });
        });
        return this;
    }
    toSeriesCategories() {
        let series = {};
        let categories = [];
        for (let r in this.result) {
            let values = [];
            for (let c in this.result[r]) {
                series[c] = {
                    label: c
                };
                values.push(this.result[r][c]);
            }
            categories.push({
                label: r,
                values: values
            });
        }
        return {
            series: Object.values(series),
            categories: categories
        };
    }
    setNestedClause(clause, id) {
        if (!clause.AND) {
            clause.AND = this.clauseCallback(id);
        }
        else {
            this.setNestedClause(clause.AND, id);
        }
    }
    getTagIds(tags) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in tags) {
                let tag = tags[i];
                yield this.setCategoryId(tag);
                yield this.setTagId(tag);
            }
            return tags.map(tag => tag.id);
        });
    }
    setCategoryId(tag) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tag.categoryId)
                return;
            const categoryItem = yield this.prisma.category.findFirst({
                where: {
                    name: tag.category
                }
            });
            if (!categoryItem) {
                throw new Error(`Category not found: ${tag.category}`);
            }
            tag.categoryId = categoryItem.id;
        });
    }
    setTagId(tag) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tag.id)
                return;
            const tagItem = yield this.prisma.tag.findFirst({
                where: {
                    categoryId: tag.categoryId,
                    name: tag.value
                }
            });
            if (!tagItem) {
                throw new Error(`Tag not found: ${tag.value} @ category: ${tag.category}`);
            }
            tag.id = tagItem.id;
        });
    }
}
exports.Query = Query;
//# sourceMappingURL=query.js.map